# -*- coding: utf-8 -*-
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.test import TestCase, Client
from compat import import_string, unquote_plus

from hijack import settings as hijack_settings
from hijack.helpers import is_authorized
from hijack.signals import hijack_started, hijack_ended
from hijack.templatetags.hijack_tags import can_hijack
from hijack.tests.utils import SettingsOverride


class BaseHijackTests(TestCase):

    def setUp(self):
        self.superuser_username = 'superuser'
        self.superuser_email = 'superuser@example.com'
        self.superuser_password = 'superuser_pw'
        self.superuser = User.objects.create_superuser(self.superuser_username, self.superuser_email, self.superuser_password)

        self.staff_user_username = 'staff_user'
        self.staff_user_email = 'staff_user@example.com'
        self.staff_user_password = 'staff_user_pw'
        self.staff_user = User.objects.create_user(self.staff_user_username, self.staff_user_email, self.staff_user_password)
        self.staff_user.is_staff = True
        self.staff_user.save()

        self.user_username = 'user'
        self.user_email = 'user@example.com'
        self.user_password = 'user_pw'
        self.user = User.objects.create_user(self.user_username, self.user_email, self.user_password)

        self.client.login(username=self.superuser_username, password=self.superuser_password)

    def tearDown(self):
        self.client.logout()

    def _hijack(self, user):
        return self.client.post('/hijack/%d/' % user.id, follow=True)

    def _release_hijack(self):
        response = self.client.post('/hijack/release-hijack/', follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertFalse('hijacked-warning' in str(response.content))
        return response


class HijackTests(BaseHijackTests):

    def setUp(self):
        super(HijackTests, self).setUp()

    def tearDown(self):
        super(HijackTests, self).tearDown()

    def test_basic_hijack(self):
        client = Client()
        client.login(username=self.superuser_username, password=self.superuser_password)
        hijacked_response = client.post('/hijack/%d/' % self.user.id, follow=True)
        self.assertEqual(hijacked_response.status_code, 200)
        hijack_released_response = client.post('/hijack/release-hijack/', follow=True)
        self.assertEqual(hijack_released_response.status_code, 200)
        client.logout()

    def assertHijackSuccess(self, response):
        self.assertEqual(response.status_code, 200)
        self.assertTrue(self.client.session['is_hijacked_user'])
        self.assertTrue('hijacked-warning' in str(response.content))
        self.assertFalse('Log in' in str(response.content))

    def assertHijackPermissionDenied(self, response):
        self.assertEqual(response.status_code, 403)
        self.assertFalse(getattr(self.client.session, 'is_hijacked_user', False))
        self.assertFalse('hijacked-warning' in str(response.content))

    def test_hijack_urls(self):
        self.assertEqual('/hijack/disable-hijack-warning/', reverse('disable_hijack_warning'))
        self.assertEqual('/hijack/release-hijack/', reverse('release_hijack'))
        self.assertEqual('/hijack/1/', reverse('login_with_id', args=[1]))
        self.assertEqual('/hijack/2/', reverse('login_with_id', kwargs={'user_id': 2}))
        self.assertEqual('/hijack/username/bob/', reverse('login_with_username', args=['bob']))
        self.assertEqual('/hijack/username/bob_too/', reverse('login_with_username', kwargs={'username': 'bob_too'}))
        self.assertEqual('/hijack/email/bob@bobsburgers.com/', unquote_plus(reverse('login_with_email', args=['bob@bobsburgers.com'])))
        self.assertEqual('/hijack/email/bob_too@bobsburgers.com/', unquote_plus(reverse('login_with_email', kwargs={'email': 'bob_too@bobsburgers.com'})))

    def test_hijack_url_user_id(self):
        response = self.client.post('/hijack/%d/' % self.user.id, follow=True)
        self.assertHijackSuccess(response)
        self._release_hijack()
        response = self.client.post('/hijack/%s/' % self.user.username, follow=True)
        self.assertEqual(response.status_code, 400)
        response = self.client.post('/hijack/-1/', follow=True)
        self.assertEqual(response.status_code, 404)

    def test_hijack_url_username(self):
        response = self.client.post('/hijack/username/%s/' % self.user_username, follow=True)
        self.assertHijackSuccess(response)
        self._release_hijack()
        response = self.client.post('/hijack/username/dfjakhdl/', follow=True)
        self.assertEqual(response.status_code, 404)

    def test_hijack_url_email(self):
        response = self.client.post('/hijack/email/%s/' % self.user_email, follow=True)
        self.assertHijackSuccess(response)
        self._release_hijack()
        response = self.client.post('/hijack/email/dfjak@hdl.com/', follow=True)
        self.assertEqual(response.status_code, 404)

    def test_hijack_permission_denied(self):
        self.client.logout()
        self.client.login(username=self.staff_user_username, password=self.staff_user_password)
        response = self._hijack(self.superuser)
        self.assertHijackPermissionDenied(response)
        response = self._hijack(self.staff_user)
        self.assertHijackPermissionDenied(response)
        response = self._hijack(self.user)
        self.assertHijackPermissionDenied(response)

    def test_release_before_hijack(self):
        response = self.client.post('/hijack/release-hijack/', follow=True)
        self.assertHijackPermissionDenied(response)

    def test_last_login_time_not_changed(self):
        self.client.logout()
        self.client.login(username=self.user_username, password=self.user_password)
        self.client.logout()
        last_non_hijack_login = User.objects.get(id=self.user.id).last_login
        self.assertIsNotNone(last_non_hijack_login)
        self.client.login(username=self.superuser_username, password=self.superuser_password)
        response = self._hijack(self.user)
        self.assertHijackSuccess(response)
        self._release_hijack()
        self.assertEqual(User.objects.get(id=self.user.id).last_login, last_non_hijack_login)

    def test_disable_hijack_warning(self):
        response = self._hijack(self.user)
        self.assertTrue('hijacked-warning' in str(response.content))
        self.assertTrue(self.client.session['is_hijacked_user'])
        self.assertTrue(self.client.session['display_hijack_warning'])

        response = self.client.post('/hijack/disable-hijack-warning/?next=/hello/', follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertFalse('hijacked-warning' in str(response.content))
        self.assertTrue(self.client.session['is_hijacked_user'])
        self.assertFalse(self.client.session['display_hijack_warning'])
        self._release_hijack()

    def test_permissions(self):
        self.assertTrue(self.superuser.is_superuser)
        self.assertTrue(self.superuser.is_staff)
        self.assertFalse(self.staff_user.is_superuser)
        self.assertTrue(self.staff_user.is_staff)
        self.assertFalse(self.user.is_superuser)
        self.assertFalse(self.user.is_staff)

    def test_settings(self):
        self.assertTrue(hasattr(hijack_settings, 'HIJACK_DISPLAY_WARNING'))
        self.assertTrue(hijack_settings.HIJACK_DISPLAY_WARNING)
        self.assertTrue(hasattr(hijack_settings, 'HIJACK_URL_ALLOWED_ATTRIBUTES'))
        self.assertEqual(hijack_settings.HIJACK_URL_ALLOWED_ATTRIBUTES, ('user_id', 'email', 'username'))
        self.assertTrue(hasattr(hijack_settings, 'HIJACK_AUTHORIZE_STAFF'))
        self.assertFalse(hijack_settings.HIJACK_AUTHORIZE_STAFF)
        self.assertTrue(hasattr(hijack_settings, 'HIJACK_AUTHORIZE_STAFF_TO_HIJACK_STAFF'))
        self.assertFalse(hijack_settings.HIJACK_AUTHORIZE_STAFF_TO_HIJACK_STAFF)
        self.assertTrue(hasattr(hijack_settings, 'HIJACK_LOGIN_REDIRECT_URL'))
        self.assertEqual(hijack_settings.HIJACK_LOGIN_REDIRECT_URL, '/hello/')
        self.assertTrue(hasattr(hijack_settings, 'HIJACK_LOGOUT_REDIRECT_URL'))
        self.assertEqual(hijack_settings.HIJACK_LOGOUT_REDIRECT_URL, '/hello/')
        self.assertTrue(hasattr(hijack_settings, 'HIJACK_AUTHORIZATION_CHECK'))
        self.assertEqual(hijack_settings.HIJACK_AUTHORIZATION_CHECK, 'hijack.helpers.is_authorized_default')
        self.assertTrue(hasattr(hijack_settings, 'HIJACK_DECORATOR'))
        self.assertEqual(hijack_settings.HIJACK_DECORATOR, 'django.contrib.admin.views.decorators.staff_member_required')
        self.assertTrue(hasattr(hijack_settings, 'HIJACK_USE_BOOTSTRAP'))
        self.assertFalse(hijack_settings.HIJACK_USE_BOOTSTRAP)
        self.assertTrue(hasattr(hijack_settings, 'HIJACK_ALLOW_GET_REQUESTS'))
        self.assertFalse(hijack_settings.HIJACK_ALLOW_GET_REQUESTS)

    def test_settings_override(self):
        self.assertFalse(hijack_settings.HIJACK_AUTHORIZE_STAFF)
        with SettingsOverride(hijack_settings, HIJACK_AUTHORIZE_STAFF=True):
            self.assertTrue(hijack_settings.HIJACK_AUTHORIZE_STAFF)
        self.assertFalse(hijack_settings.HIJACK_AUTHORIZE_STAFF)

    def test_is_authorized(self):
        self.assertFalse(hijack_settings.HIJACK_AUTHORIZE_STAFF)
        self.assertFalse(hijack_settings.HIJACK_AUTHORIZE_STAFF_TO_HIJACK_STAFF)
        self.assertTrue(is_authorized(self.superuser, self.superuser))
        self.assertTrue(is_authorized(self.superuser, self.staff_user))
        self.assertTrue(is_authorized(self.superuser, self.user))
        self.assertFalse(is_authorized(self.staff_user, self.superuser))
        self.assertFalse(is_authorized(self.staff_user, self.staff_user))
        self.assertFalse(is_authorized(self.staff_user, self.user))
        self.assertFalse(is_authorized(self.user, self.superuser))
        self.assertFalse(is_authorized(self.user, self.staff_user))
        self.assertFalse(is_authorized(self.user, self.user))

    def test_is_authorized_staff_authorized(self):
        with SettingsOverride(hijack_settings, HIJACK_AUTHORIZE_STAFF=True):
            self.assertTrue(hijack_settings.HIJACK_AUTHORIZE_STAFF)
            self.assertFalse(hijack_settings.HIJACK_AUTHORIZE_STAFF_TO_HIJACK_STAFF)
            self.assertTrue(is_authorized(self.superuser, self.superuser))
            self.assertTrue(is_authorized(self.superuser, self.staff_user))
            self.assertTrue(is_authorized(self.superuser, self.user))
            self.assertFalse(is_authorized(self.staff_user, self.superuser))
            self.assertFalse(is_authorized(self.staff_user, self.staff_user))
            self.assertTrue(is_authorized(self.staff_user, self.user))
            self.assertFalse(is_authorized(self.user, self.superuser))
            self.assertFalse(is_authorized(self.user, self.staff_user))
            self.assertFalse(is_authorized(self.user, self.user))

    def test_is_authorized_staff_authorized_to_hijack_staff(self):
        with SettingsOverride(hijack_settings, HIJACK_AUTHORIZE_STAFF=True, HIJACK_AUTHORIZE_STAFF_TO_HIJACK_STAFF=True):
            self.assertTrue(hijack_settings.HIJACK_AUTHORIZE_STAFF)
            self.assertTrue(hijack_settings.HIJACK_AUTHORIZE_STAFF_TO_HIJACK_STAFF)
            self.assertTrue(is_authorized(self.superuser, self.superuser))
            self.assertTrue(is_authorized(self.superuser, self.staff_user))
            self.assertTrue(is_authorized(self.superuser, self.user))
            self.assertFalse(is_authorized(self.staff_user, self.superuser))
            self.assertTrue(is_authorized(self.staff_user, self.staff_user))
            self.assertTrue(is_authorized(self.staff_user, self.user))
            self.assertFalse(is_authorized(self.user, self.superuser))
            self.assertFalse(is_authorized(self.user, self.staff_user))
            self.assertFalse(is_authorized(self.user, self.user))

    def test_disallow_get_requests(self):
        self.assertFalse(hijack_settings.HIJACK_ALLOW_GET_REQUESTS)
        protected_urls = [
            '/hijack/{}/'.format(self.user.id),
            '/hijack/email/{}/'.format(self.user_email),
            '/hijack/username/{}/'.format(self.user_username),
            '/hijack/disable-hijack-warning/',
            '/hijack/release-hijack/',
        ]
        for protected_url in protected_urls:
            self.assertEqual(self.client.get(protected_url, follow=True).status_code, 405,
                             msg='GET requests should not be allowed')

    def test_notification_tag(self):
        response = self._hijack(self.user)
        self.assertHijackSuccess(response)
        response = self.client.get('/hello/')
        self.assertEqual(response.status_code, 200)
        self.assertTrue('Notification tag' in str(response.content))
        self.assertTrue('hijacked-warning' in str(response.content))

    def test_is_hijacked_filter(self):
        response = self._hijack(self.user)
        self.assertHijackSuccess(response)
        response = self.client.get('/hello/')
        print(response.content)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'You are hijacking this user')

    def test_notification_filter(self):
        response = self._hijack(self.user)
        self.assertHijackSuccess(response)
        response = self.client.get('/hello/filter/')
        self.assertEqual(response.status_code, 200)
        self.assertTrue('Notification filter' in str(response.content))
        self.assertTrue('hijacked-warning' in str(response.content))

    def test_bootstrap_option(self):
        with SettingsOverride(hijack_settings, HIJACK_USE_BOOTSTRAP=True):
            response = self._hijack(self.user)
            self.assertHijackSuccess(response)
            response = self.client.get('/hello/')
            self.assertTrue('hijacked-warning-bootstrap' in str(response.content))
            response = self.client.get('/hello/filter/')
            self.assertTrue('hijacked-warning-bootstrap' in str(response.content))

    def test_can_hijack_filter(self):
        self.assertTrue(can_hijack(self.superuser, self.staff_user))
        self.assertFalse(can_hijack(self.user, self.superuser))

    def test_custom_authorization_check(self):
        for custom_check_path in [
            'hijack.tests.test_app.authorization_checks.can_hijack_default',
            'hijack.tests.test_app.authorization_checks.everybody_can_hijack',
            'hijack.tests.test_app.authorization_checks.nobody_can_hijack',
        ]:
            with SettingsOverride(hijack_settings, HIJACK_AUTHORIZATION_CHECK=custom_check_path):
                custom_check = import_string(custom_check_path)
                for hijacker, hijacked in [
                        (self.superuser, self.superuser),
                        (self.superuser, self.staff_user),
                        (self.superuser, self.user),
                        (self.staff_user, self.superuser),
                        (self.staff_user, self.staff_user),
                        (self.staff_user, self.user),
                        (self.user, self.superuser),
                        (self.user, self.staff_user),
                        (self.user, self.user),
                ]:
                    self.assertEqual(custom_check(hijacker, hijacked), is_authorized(hijacker, hijacked))

    def test_default_decorator(self):
        self.client.logout()
        self.client.login(username=self.user_username, password=self.user_password)
        response = self._hijack(self.staff_user)
        self.assertEqual(response.status_code, 200)
        self.assertIn('Log in', str(response.content))

    def test_signals(self):
        received_signals = []

        def hijack_started_receiver(sender, hijacker_id, hijacked_id, request, **kwargs):
            received_signals.append('hijack_started_%d_%d' % (hijacker_id, hijacked_id))
        hijack_started.connect(hijack_started_receiver)

        def hijack_ended_receiver(sender, hijacker_id, hijacked_id, request, **kwargs):
            received_signals.append('hijack_ended_%d_%d' % (hijacker_id, hijacked_id))
        hijack_ended.connect(hijack_ended_receiver)

        self.assertEqual(len(received_signals), 0)
        self._hijack(self.user)
        self.assertEqual(len(received_signals), 1)
        self.assertIn('hijack_started_%d_%d' % (self.superuser.id, self.user.id), received_signals)
        self._release_hijack()
        self.assertEqual(len(received_signals), 2)
        self.assertIn('hijack_ended_%d_%d' % (self.superuser.id, self.user.id), received_signals)



import django
from django import template
from django.utils.safestring import mark_safe
from django.template.loader import render_to_string
from compat import import_string

from hijack import settings as hijack_settings

register = template.Library()


@register.filter
def hijackNotification(request):
    """
    Deprecated. Use the template tag "hijack_notification" below
    """
    return _render_hijack_notification(request)


@register.simple_tag(takes_context=True)
def hijack_notification(context):
    request = context.get('request')
    return _render_hijack_notification(request)


def _render_hijack_notification(request):
    if hijack_settings.HIJACK_USE_BOOTSTRAP:
        template_name = 'hijack/notifications_bootstrap.html'
    else:
        template_name = 'hijack/notifications.html'
    ans = ''
    if request is not None and all([
        hijack_settings.HIJACK_DISPLAY_WARNING,
        request.session.get('is_hijacked_user', False),
        request.session.get('display_hijack_warning', False),
    ]):
        if django.VERSION < (1, 8):
            from django.template import RequestContext
            ans = render_to_string(template_name, context_instance=RequestContext(request))
        else:
            ans = render_to_string(template_name, request=request)
    return mark_safe(ans)


@register.filter
def can_hijack(hijacker, hijacked):
    check_authorization = import_string(hijack_settings.HIJACK_AUTHORIZATION_CHECK)
    return check_authorization(hijacker, hijacked)


@register.filter
def is_hijacked(request):
    return request.session.get('is_hijacked_user', False)



# -*- coding: utf-8 -*-
from django.conf import settings as django_settings

SETTINGS = (
    {
        'name': 'HIJACK_DISPLAY_WARNING',
        'default': True,
        'legacy_name': 'HIJACK_NOTIFY_ADMIN',
    },
    {
        'name': 'HIJACK_URL_ALLOWED_ATTRIBUTES',
        'default': ('user_id', 'email', 'username'),
        'legacy_name': 'ALLOWED_HIJACKING_USER_ATTRIBUTES',
    },
    {
        'name': 'HIJACK_AUTHORIZE_STAFF',
        'default': False,
        'legacy_name': 'ALLOW_STAFF_TO_HIJACKUSER',
    },
    {
        'name': 'HIJACK_AUTHORIZE_STAFF_TO_HIJACK_STAFF',
        'default': False,
        'legacy_name': 'ALLOW_STAFF_TO_HIJACK_STAFF_USER',
    },
    {
        'name': 'HIJACK_ALLOW_GET_REQUESTS',
        'default': False,
        'legacy_name': None,
    },
    {
        'name': 'HIJACK_LOGIN_REDIRECT_URL',
        'default': getattr(django_settings, 'LOGIN_REDIRECT_URL', '/'),
        'legacy_name': None,
    },
    {
        'name': 'HIJACK_LOGOUT_REDIRECT_URL',
        'default': getattr(django_settings, 'LOGIN_REDIRECT_URL', '/'),
        'legacy_name': 'REVERSE_HIJACK_LOGIN_REDIRECT_URL',
    },
    {
        'name': 'HIJACK_AUTHORIZATION_CHECK',
        'default': 'hijack.helpers.is_authorized_default',
        'legacy_name': 'CUSTOM_HIJACK_HANDLER',
    },
    {
        'name': 'HIJACK_DECORATOR',
        'default': 'django.contrib.admin.views.decorators.staff_member_required',
        'legacy_name': None,
    },
    {
        'name': 'HIJACK_USE_BOOTSTRAP',
        'default': False,
        'legacy_name': None,
    },
)

for setting in SETTINGS:
    if setting['legacy_name']:
        default = getattr(django_settings, setting['legacy_name'], setting['default'])
    else:
        default = setting['default']
    value = getattr(django_settings, setting['name'], default)
    globals()[setting['name']] = value



# -*- coding: utf-8 -*-
from django.apps import AppConfig


class HijackConfig(AppConfig):
    name = 'hijack'
    verbose_name = 'Hijack'

    def ready(self):
        from hijack.checks import register_checks
        register_checks()



from django.dispatch import Signal

hijack_started = Signal(providing_args=['hijacker_id', 'hijacked_id', 'request'])
hijack_ended = Signal(providing_args=['hijacker_id', 'hijacked_id', 'request'])

# Deprecated signals
post_superuser_login = Signal(providing_args=['user_id'])
post_superuser_logout = Signal(providing_args=['user_id'])



# -*- encoding: utf-8 -*-
from django.contrib.auth.decorators import login_required
from django.http import HttpResponseBadRequest
from django.shortcuts import get_object_or_404

from hijack.decorators import hijack_require_http_methods, hijack_decorator
from hijack.helpers import login_user, redirect_to_next
from hijack.helpers import release_hijack as release_hijack_fx

from compat import get_user_model


@hijack_decorator
@hijack_require_http_methods
def login_with_id(request, user_id):
    # input(user_id) is unicode
    try:
        user_id = int(user_id)
    except ValueError:
        return HttpResponseBadRequest('user_id must be an integer value.')
    user = get_object_or_404(get_user_model(), pk=user_id)
    return login_user(request, user)


@hijack_decorator
@hijack_require_http_methods
def login_with_email(request, email):
    user = get_object_or_404(get_user_model(), email=email)
    return login_user(request, user)


@hijack_decorator
@hijack_require_http_methods
def login_with_username(request, username):
    user = get_object_or_404(get_user_model(), username=username)
    return login_user(request, user)


@login_required
@hijack_require_http_methods
def release_hijack(request):
    return release_hijack_fx(request)


@login_required
@hijack_require_http_methods
def disable_hijack_warning(request):
    request.session['display_hijack_warning'] = False
    return redirect_to_next(request, default_url='/')



class HijackRemoteUserMiddleware(object):
    """
    Middleware for hijack RemoteUser. One must place this middleware between
    'django.contrib.auth.middleware.AuthenticationMiddleware' and
    'django.contrib.auth.middleware.RemoteUserMiddleware' in MIDDLEWARE_CLASSES

    Just makes remote user same as hijacked
    """
    header = "REMOTE_USER"

    def process_request(self, request):
        is_hijacked = request.session.get('is_hijacked_user', False)
        remote_username = request.META.get(self.header, None)
        if not is_hijacked or not remote_username:
            return
        # Ok, we hijacked and remote. Just assign hijacked user to remote
        if request.user.is_authenticated():
            username = request.user.get_username()
            if username != remote_username:
                request.META[self.header] = username
                
    def authenticate(self, *args, **kwargs):
        return None



# -*- coding: utf-8 -*-
from django.core.exceptions import PermissionDenied
from django.contrib.auth.models import update_last_login
from django.contrib.auth.signals import user_logged_in, user_logged_out
from django.contrib.auth import login, load_backend, BACKEND_SESSION_KEY
from django.dispatch import receiver
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.utils.http import is_safe_url

from compat import get_user_model, import_string
from compat import resolve_url

from hijack import settings as hijack_settings
from hijack.signals import post_superuser_login, post_superuser_logout, hijack_started, hijack_ended


def get_used_backend(request):
    backend_str = request.session[BACKEND_SESSION_KEY]
    backend = load_backend(backend_str)
    return backend


def release_hijack(request):
    hijack_history = request.session.get('hijack_history', False)

    if not hijack_history:
        raise PermissionDenied

    hijacker = None
    hijacked = None
    if hijack_history:
        hijacked = request.user
        user_pk = hijack_history.pop()
        hijacker = get_object_or_404(get_user_model(), pk=user_pk)
        backend = get_used_backend(request)
        hijacker.backend = "%s.%s" % (backend.__module__, backend.__class__.__name__)
        login(request, hijacker)
    if hijack_history:
        request.session['hijack_history'] = hijack_history
        request.session['is_hijacked_user'] = True
        request.session['display_hijack_warning'] = True
    else:
        request.session.pop('hijack_history', None)
        request.session.pop('is_hijacked_user', None)
        request.session.pop('display_hijack_warning', None)
    request.session.modified = True
    hijack_ended.send(sender=None, hijacker_id=hijacker.pk, hijacked_id=hijacked.pk, request=request)
    return redirect_to_next(request, default_url=hijack_settings.HIJACK_LOGOUT_REDIRECT_URL)


def is_authorized_default(hijacker, hijacked):
    """Checks if the user has the correct permission to Hijack another user.

    By default only superusers are allowed to hijack.

    An exception is made to allow staff members to hijack when
    HIJACK_AUTHORIZE_STAFF is enabled in the Django settings.

    By default it prevents staff users from hijacking other staff users.
    This can be disabled by enabling the HIJACK_AUTHORIZE_STAFF_TO_HIJACK_STAFF
    setting in the Django settings.

    Staff users can never hijack superusers.
    """

    if hijacker.is_superuser:
        return True

    if hijacked.is_superuser:
        return False

    if hijacker.is_staff and hijack_settings.HIJACK_AUTHORIZE_STAFF:
        if hijacked.is_staff and not hijack_settings.HIJACK_AUTHORIZE_STAFF_TO_HIJACK_STAFF:
            return False
        return True

    return False


def is_authorized(hijack, hijacked):
    '''
    Evaluates the authorization check specified in settings
    '''
    authorization_check = import_string(hijack_settings.HIJACK_AUTHORIZATION_CHECK)
    return authorization_check(hijack, hijacked)


def check_hijack_authorization(request, user):
    if not is_authorized(request.user, user):
        raise PermissionDenied


def login_user(request, hijacked):
    ''' hijack mechanism '''
    hijacker = request.user
    hijack_history = [request.user._meta.pk.value_to_string(hijacker)]
    if request.session.get('hijack_history'):
        hijack_history = request.session['hijack_history'] + hijack_history

    check_hijack_authorization(request, hijacked)

    backend = get_used_backend(request)
    hijacked.backend = "%s.%s" % (backend.__module__, backend.__class__.__name__)

    # Prevent update of hijacked user last_login
    signal_was_connected = user_logged_in.disconnect(update_last_login)

    # Actually log user in
    login(request, hijacked)

    # Restore signal if needed
    if signal_was_connected:
        user_logged_in.connect(update_last_login)

    post_superuser_login.send(sender=None, user_id=hijacked.pk)  # Send legacy signal
    hijack_started.send(sender=None, hijacker_id=hijacker.pk, hijacked_id=hijacked.pk, request=request)  # Send official, documented signal
    request.session['hijack_history'] = hijack_history
    request.session['is_hijacked_user'] = True
    request.session['display_hijack_warning'] = True
    request.session.modified = True
    return redirect_to_next(request, default_url=hijack_settings.HIJACK_LOGIN_REDIRECT_URL)


@receiver(user_logged_out)
def logout_user(sender, **kwargs):
    """
    Legacy code
    Wraps logout signal to send deprecated "post_superuser_logout" signal
    """
    user = kwargs['user']
    if hasattr(user, 'id'):
        post_superuser_logout.send(sender=None, user_id=user.pk)


def redirect_to_next(request, default_url=hijack_settings.HIJACK_LOGIN_REDIRECT_URL):
    redirect_to = request.GET.get('next', '')
    if not is_safe_url(redirect_to):
        redirect_to = default_url
    return HttpResponseRedirect(resolve_url(redirect_to))



# -*- encoding: utf-8 -*-
from compat import import_string
from django.views.decorators.http import require_http_methods

from hijack import settings as hijack_settings


def hijack_decorator(fn):
    """
    Apply customizable decorator to sensitive methods. Default: staff_member_required
    """
    decorator = import_string(hijack_settings.HIJACK_DECORATOR)
    return decorator(fn)


def hijack_require_http_methods(fn):
    """
    Wrapper for "require_http_methods" decorator. POST required by default, GET can optionally be allowed
    """
    required_methods = ['POST']
    if hijack_settings.HIJACK_ALLOW_GET_REQUESTS:
        required_methods.append('GET')
    return require_http_methods(required_methods)(fn)



# -*- coding: utf-8 -*-
# Test cases for settings-dependent urls/decorators. Need to be run separately from HijackTests

from unittest import skipIf
import sys

from hijack import settings as hijack_settings
from hijack.tests.test_hijack import BaseHijackTests
from hijack.tests.utils import SettingsOverride


class AllowGetRequestsHijackTestCase(BaseHijackTests):

    def setUp(self):
        super(AllowGetRequestsHijackTestCase, self).setUp()

    def tearDown(self):
        super(AllowGetRequestsHijackTestCase, self).tearDown()

    @skipIf('AllowGetRequestsHijackTestCase' not in ' '.join(sys.argv),
            'Skipping test cases for settings-dependent urls/decorators which need to be run separately')
    def test_allow_get_requests(self):
        protected_urls = [
            '/hijack/{}/'.format(self.user.id),
            '/hijack/email/{}/'.format(self.user_email),
            '/hijack/username/{}/'.format(self.user_username),
            '/hijack/disable-hijack-warning/',
            '/hijack/release-hijack/',
        ]

        with SettingsOverride(hijack_settings, HIJACK_ALLOW_GET_REQUESTS=True):
            self.assertTrue(hijack_settings.HIJACK_ALLOW_GET_REQUESTS)
            for protected_url in protected_urls:
                self.assertNotEqual(self.client.get(protected_url, follow=True).status_code, 405,
                                    msg='GET requests should be allowed')


class CustomDecoratorHijackTestCase(BaseHijackTests):

    def setUp(self):
        super(CustomDecoratorHijackTestCase, self).setUp()

    def tearDown(self):
        super(CustomDecoratorHijackTestCase, self).tearDown()

    @skipIf('CustomDecoratorHijackTestCase' not in ' '.join(sys.argv),
            'Skipping test cases for settings-dependent urls/decorators which need to be run separately')
    def test_custom_decorator(self):
        custom_decorator_path = 'hijack.tests.test_app.decorators.no_decorator'
        with SettingsOverride(hijack_settings, HIJACK_DECORATOR=custom_decorator_path):
            self.assertEqual(hijack_settings.HIJACK_DECORATOR, custom_decorator_path)
            self.client.logout()
            self.client.login(username=self.user_username, password=self.user_password)
            response = self._hijack(self.staff_user)
            self.assertEqual(response.status_code, 403)
            self.assertNotIn('Log in', str(response.content))




# Borrowed from https://github.com/stefanfoulis/django-filer/blob/develop/filer/tests/helpers.py
class SettingsOverride(object):
    """
    Overrides Django settings within a context and resets them to their inital
    values on exit.
    Example:
        with SettingsOverride(DEBUG=True):
            # do something
    """

    def __init__(self, settings_module, **overrides):
        self.settings_module = settings_module
        self.overrides = overrides

    def __enter__(self):
        self.old = {}
        for key, value in list(self.overrides.items()):
            self.old[key] = getattr(self.settings_module, key, None)
            setattr(self.settings_module, key, value)

    def __exit__(self, _type, value, traceback):
        for key, value in list(self.old.items()):
            if value is not None:
                setattr(self.settings_module, key, value)
            else:
                delattr(self.settings_module, key)


from compat import url

from hijack import settings as hijack_settings
from hijack import views

urlpatterns = [
    url(
        r'^release-hijack/$',
        views.release_hijack,
        name='release_hijack'
    )
]

if hijack_settings.HIJACK_DISPLAY_WARNING:
    urlpatterns.append(url(
        r'^disable-hijack-warning/$',
        views.disable_hijack_warning,
        name='disable_hijack_warning'
    ))

hijacking_user_attributes = hijack_settings.HIJACK_URL_ALLOWED_ATTRIBUTES
if 'email' in hijacking_user_attributes:
    urlpatterns.append(url(
        r'^email/(?P<email>[^@]+@[^@]+\.[^@]+)/$',
        views.login_with_email,
        name='login_with_email'
    ))
if 'username' in hijacking_user_attributes:
    urlpatterns.append(url(
        r'^username/(?P<username>.*)/$',
        views.login_with_username,
        name='login_with_username'
    ))
if 'user_id' in hijacking_user_attributes:
    urlpatterns.append(url(
        r'^(?P<user_id>[\w-]+)/$',
        views.login_with_id,
        name='login_with_id'
    ))



# -*- coding: utf-8 -*-
from django.core.checks import Error, Warning, register
from django.conf import settings
from django.contrib.admin.views.decorators import staff_member_required

from compat import import_string
from hijack import settings as hijack_settings


def check_legacy_settings(app_configs, **kwargs):
    warnings = []
    for setting in hijack_settings.SETTINGS:
        legacy_name = setting['legacy_name']
        if legacy_name and hasattr(settings, legacy_name):
            warnings.append(
                Warning(
                    'Deprecation warning: Setting "%s" has been renamed to "%s"'
                    % (legacy_name, setting['name']),
                    hint=None,
                    obj=None,
                    id='hijack.W002',
                )
            )
    return warnings


def check_url_allowed_attributes(app_configs, **kwargs):
    errors = []
    required_attributes = [
        'user_id',
        'email',
        'username',
    ]
    set_attributes = hijack_settings.HIJACK_URL_ALLOWED_ATTRIBUTES
    if not any(attribute in set_attributes for attribute in required_attributes):
        errors.append(
            Error(
                'Setting HIJACK_URL_ALLOWED_ATTRIBUTES needs to be subset of (%s)'
                    % ', '.join(required_attributes),
                hint=None,
                obj=set_attributes,
                id='hijack.E001',
            )
        )
    return errors


def check_custom_authorization_check_importable(app_configs, **kwargs):
    errors = []
    authorization_check = hijack_settings.HIJACK_AUTHORIZATION_CHECK
    try:
        if authorization_check != staff_member_required:
            import_string(authorization_check)
    except ImportError:
        errors.append(
            Error(
                'Setting HIJACK_AUTHORIZATION_CHECK cannot be imported',
                hint=None,
                obj=authorization_check,
                id='hijack.E002',
            )
        )
    return errors


def check_hijack_decorator_importable(app_configs, **kwargs):
    errors = []
    decorator = hijack_settings.HIJACK_DECORATOR
    try:
        if decorator != 'django.contrib.admin.views.decorators.staff_member_required':
            import_string(decorator)
    except ImportError:
        errors.append(
            Error(
                'Setting HIJACK_DECORATOR cannot be imported',
                hint=None,
                obj=decorator,
                id='hijack.E003',
            )
        )
    return errors


def check_staff_authorization_settings(app_configs, **kwargs):
    errors = []
    if hijack_settings.HIJACK_AUTHORIZE_STAFF_TO_HIJACK_STAFF and not hijack_settings.HIJACK_AUTHORIZE_STAFF:
        errors.append(
            Error(
                'Setting HIJACK_AUTHORIZE_STAFF_TO_HIJACK_STAFF may not be True if HIJACK_AUTHORIZE_STAFF is False.',
                hint=None,
                obj=None,
                id='hijack.E004',
            )
        )
    return errors


def register_checks():
    for check in [
        check_legacy_settings,
        check_url_allowed_attributes,
        check_custom_authorization_check_importable,
        check_hijack_decorator_importable,
        check_staff_authorization_settings,
    ]:
        register(check)



# -*- coding: utf-8 -*-
import django

if django.VERSION < (1, 7):
    pass
else:
    from django.conf import settings
    from django.core.checks import Error, Warning
    from django.test import TestCase

    from hijack import checks
    from hijack import settings as hijack_settings
    from hijack.apps import HijackConfig
    from hijack.tests.utils import SettingsOverride


    class ChecksTests(TestCase):

        def test_check_legacy_settings(self):
            with SettingsOverride(settings, ALLOW_STAFF_TO_HIJACKUSER=True):
                warnings = checks.check_legacy_settings(HijackConfig)
                expected_warnings = [
                    Warning(
                        'Deprecation warning: Setting "ALLOW_STAFF_TO_HIJACKUSER" has been renamed to "HIJACK_AUTHORIZE_STAFF"',
                        hint=None,
                        obj=None,
                        id='hijack.W002'
                    )
                ]
                self.assertEqual(warnings, expected_warnings)

        def test_check_url_allowed_attributes(self):
            errors = checks.check_url_allowed_attributes(HijackConfig)
            self.assertFalse(errors)

            with SettingsOverride(hijack_settings, HIJACK_URL_ALLOWED_ATTRIBUTES=('username',)):
                errors = checks.check_url_allowed_attributes(HijackConfig)
                self.assertFalse(errors)

            with SettingsOverride(hijack_settings, HIJACK_URL_ALLOWED_ATTRIBUTES=('username', 'email')):
                errors = checks.check_url_allowed_attributes(HijackConfig)
                self.assertFalse(errors)

            with SettingsOverride(hijack_settings, HIJACK_URL_ALLOWED_ATTRIBUTES=('other',)):
                errors = checks.check_url_allowed_attributes(HijackConfig)
                expected_errors = [
                    Error(
                        'Setting HIJACK_URL_ALLOWED_ATTRIBUTES needs to be '
                        'subset of (user_id, email, username)',
                        hint=None,
                        obj=hijack_settings.HIJACK_URL_ALLOWED_ATTRIBUTES,
                        id='hijack.E001',
                    )
                ]
                self.assertEqual(errors, expected_errors)

        def test_check_custom_authorization_check_importable(self):
            errors = checks.check_custom_authorization_check_importable(HijackConfig)
            self.assertFalse(errors)
            with SettingsOverride(hijack_settings, HIJACK_AUTHORIZATION_CHECK='not.importable'):
                expected_errors = [
                    Error(
                        'Setting HIJACK_AUTHORIZATION_CHECK cannot be imported',
                        hint=None,
                        obj='not.importable',
                        id='hijack.E002',
                    )
                ]
                errors = checks.check_custom_authorization_check_importable(HijackConfig)
                self.assertEqual(errors, expected_errors)

        def test_hijack_decorator_importable(self):
            errors = checks.check_hijack_decorator_importable(HijackConfig)
            self.assertFalse(errors)
            with SettingsOverride(hijack_settings, HIJACK_DECORATOR='not.importable'):
                expected_errors = [
                    Error(
                        'Setting HIJACK_DECORATOR cannot be imported',
                        hint=None,
                        obj='not.importable',
                        id='hijack.E003',
                    )
                ]
                errors = checks.check_hijack_decorator_importable(HijackConfig)
                self.assertEqual(errors, expected_errors)

        def test_check_staff_authorization_settings(self):
            errors = checks.check_staff_authorization_settings(HijackConfig)
            self.assertFalse(errors)
            with SettingsOverride(hijack_settings, HIJACK_AUTHORIZE_STAFF=True):
                errors = checks.check_staff_authorization_settings(HijackConfig)
                self.assertFalse(errors)
            with SettingsOverride(hijack_settings, HIJACK_AUTHORIZE_STAFF=True, HIJACK_AUTHORIZE_STAFF_TO_HIJACK_STAFF=True):
                errors = checks.check_staff_authorization_settings(HijackConfig)
                self.assertFalse(errors)
            with SettingsOverride(hijack_settings, HIJACK_AUTHORIZE_STAFF=False, HIJACK_AUTHORIZE_STAFF_TO_HIJACK_STAFF=True):
                errors = checks.check_staff_authorization_settings(HijackConfig)
                expected_errors = [
                    Error(
                        'Setting HIJACK_AUTHORIZE_STAFF_TO_HIJACK_STAFF may not be True if HIJACK_AUTHORIZE_STAFF is False.',
                        hint=None,
                        obj=None,
                        id='hijack.E004',
                    )
                ]
                self.assertEqual(errors, expected_errors)